/* MF_Alarm.h */

#ifndef MF_ALARM_H
#define MF_ALARM_H

#include "MF.h"
#include "MF_Date.h"
#include "MF_Time.h"
#include "MF_TimeMgr.h"

enum AlarmType{ MF_ALARM_PERIODIC,
                MF_ALARM_MONTHLY,
                MF_ALARM_YEARLY };

typedef enum AlarmType MF_AlarmType;

struct AlarmClass{
  MF_AlarmType type;                /* calendar type */
  MF_TimeClass offset;              /* offset from start */
  MF_TimeClass period;              /* alarm period */
  MF_Bool alarmOn;                  /* true if on */
};

typedef struct AlarmClass *MF_Alarm;

typedef struct AlarmClass MF_AlarmClass;

/*============================================================================*
 * Public methods
 *============================================================================*/

extern MF_Alarm MF_AlarmNewPeriodic(MF_Time period, MF_Time offset);

extern MF_Alarm MF_AlarmNewMonthly();

extern MF_Alarm MF_AlarmNewYearly();

extern int MF_AlarmConstructPeriodic(MF_Alarm this, MF_Time period, 
				      MF_Time offset);

extern int MF_AlarmConstructMonthly(MF_Alarm this);

extern int MF_AlarmConstructYearly(MF_Alarm this);

extern int MF_AlarmIsOn(MF_Alarm this, MF_TimeMgr timeMgr, MF_Bool *alarmOn);

extern int MF_AlarmSet(MF_Alarm this, MF_Bool alarmOn);

extern int MF_AlarmGetType(MF_Alarm this, MF_AlarmType *type);

extern void MF_AlarmDelete(MF_Alarm this);

#endif



















