/* MF_Date.h */

#ifndef MF_DATE_H
#define MF_DATE_H
           
#include "MF.h"
#include "MF_Constants.h"
#include "MF_Calendar.h"
#include "MF_Time.h"
#include "MF_TOD.h"

struct DateClass{
  MF_CalendarClass calendar;

  int year;                         /* date year */         
  int month;                        /* date month */
  int day;                          /* date day */
  MF_TODClass tod;                  /* date time of day */           

  int julianDay;                    /* absolute day for date calculations */
  int dayOfYear;                    /* day of year (e.g. Jan 1 is day 1) */
};

typedef struct DateClass *MF_Date;

typedef struct DateClass MF_DateClass;

/*============================================================================*
 * Public methods
 *============================================================================*/

extern MF_Date MF_DateNewIS(MF_CalendarType type, int yearmmdd, int seconds);

extern MF_Date MF_DateNewUndefined();

extern int MF_DateConstructIS(MF_Date this, MF_CalendarType type, 
				int yearmmdd, int seconds);

extern int MF_DateConstructUndefined(MF_Date this);

extern int MF_DateCopy(MF_Date this, MF_Date orig);

extern int MF_DateCopyConstruct(MF_Date this, MF_Date orig);

extern int MF_DateSetIS(MF_Date this, MF_CalendarType type, int yearmmdd, int seconds);

extern int MF_DateGetIS(MF_Date this, int *yearmmdd, int *seconds);

extern int MF_DateGetCalendarType(MF_Date this, MF_CalendarType *type);

extern int MF_DateGetDayOfYear(MF_Date, int *dayOfYear);

extern int MF_Date2Str(MF_Date this, char *str);

extern int MF_DateIncrement(MF_Date this, MF_Date incDate, MF_Time time);

extern int MF_DateDecrement(MF_Date this, MF_Date decDate, MF_Time time);

extern int MF_DateIncrementSec(MF_Date this, MF_Date incDate, int seconds);

extern int MF_DateIncrementDay(MF_Date this, MF_Date incDate, int days);

extern int MF_DateIncrementMon(MF_Date this, MF_Date incDate, int months);

extern int MF_DateIncrementYear(MF_Date this, MF_Date incDate, int years);

extern int MF_DateDecrement(MF_Date this, MF_Date decDate, MF_Time time);

extern int MF_DateDiff(MF_Date earlyDate, MF_Date laterDate, 
			MF_Time diff, MF_Bool *isLater);

extern int MF_DateDiffDays(MF_Date earlyDate, MF_Date lateDate, double *days);

extern int MF_DateIsLater(MF_Date earlyDate, MF_Date lateDate, MF_Bool *isLater);

extern int MF_DatePrint(MF_Date this);

extern int MF_DateGetFltDayOfYear(MF_Date this, double *day);

extern void MF_DateDelete(MF_Date this);

#endif
