/* MF_TimeMgr.h */

#ifndef MF_TIME_MGR_H
#define MF_TIME_MGR_H

#include "MF.h"
#include "MF_Time.h"
#include "MF_Date.h"

struct TimeMgrClass{
  int nstep;
  MF_TimeClass stepSize;
  MF_DateClass startDate;
  MF_DateClass stopDate;
  MF_DateClass baseDate;
  MF_DateClass currDate;
  MF_DateClass prevDate;
};

typedef struct TimeMgrClass *MF_TimeMgr;

typedef struct TimeMgrClass MF_TimeMgrClass;

/*============================================================================*
 * Public methods
 *============================================================================*/

extern MF_TimeMgr MF_TimeMgrNew(MF_Time stepSize, MF_Date startDate, 
				MF_Date stopDate, MF_Date baseDate);

extern MF_TimeMgr MF_TimeMgrNewIS(int stepDays, int stepSecs,
				  int startCalendarDate, int startTOD,
				  int stopCalendarDate, int stopTOD,
				  int baseCalendarDate, int baseTOD,
				  MF_CalendarType type);

extern int MF_TimeMgrConstruct(MF_TimeMgr this, MF_Time stepSize, MF_Date startDate, 
				MF_Date stopDate, MF_Date baseDate);

extern int MF_TimeMgrConstructIS(MF_TimeMgr this, int stepDays, int stepSecs,
				  int startCalendarDate, int startTOD,
				  int stopCalendarDate, int stopTOD,
				  int baseCalendarDate, int baseTOD,
				  MF_CalendarType type);

extern int MF_TimeMgrConstructNoBase(MF_TimeMgr this, MF_Time stepSize, 
				      MF_Date startDate, MF_Date stopDate);

extern int MF_TimeMgrConstructNoBaseIS(MF_TimeMgr this, 
					int stepDays, int stepSecs,
					int startCalendarDate, int startTOD,
					int stopCalendarDate, int stopTOD,
					MF_CalendarType type);

extern int MF_TimeMgrAdvance(MF_TimeMgr this);

extern int MF_TimeMgrLastStep(MF_TimeMgr this, MF_Bool *lastStep);

extern int MF_TimeMgrGetNStep(MF_TimeMgr this, int *nstep);

extern int MF_TimeMgrSetNStep(MF_TimeMgr this, int nstep);

extern int MF_TimeMgrSetStepSize(MF_TimeMgr this, MF_Time stepSize);

extern int MF_TimeMgrSetStepSizeIS(MF_TimeMgr this, int days, int seconds);

extern int MF_TimeMgrGetStepSize(MF_TimeMgr this, MF_Time stepSize);

extern int MF_TimeMgrGetStepSizeIS(MF_TimeMgr this, int *days, int *seconds);

extern int MF_TimeMgrGetStartDate(MF_TimeMgr this, MF_Date startDate);

extern int MF_TimeMgrGetStopDate(MF_TimeMgr this, MF_Date stopDate);

extern int MF_TimeMgrGetBaseDate(MF_TimeMgr this, MF_Date baseDate);

extern int MF_TimeMgrGetCurrDate(MF_TimeMgr this, MF_Date currDate);

extern int MF_TimeMgrSetCurrDateIS(MF_TimeMgr this, int dateYYMMDD, int tod);

extern int MF_TimeMgrGetPrevDate(MF_TimeMgr this, MF_Date prevDate);

extern int MF_TimeMgrRestartWriteIS(MF_TimeMgr this,
				    MF_CalendarType *type,
				    int *nstep,
				    int *stepDays, int *stepSec,
				    int *startYYMMDD, int *startSec,
				    int *stopYYMMDD, int *stopSec,
				    int *baseYYMMDD, int *baseSec,
				    int *currYYMMDD, int *currSec);

extern int MF_TimeMgrRestartReadIS(MF_TimeMgr this,
				   MF_CalendarType type,
				   int nstep,
				   int stepDays, int stepSec,
				   int startYYMMDD, int startSec,
				   int stopYYMMDD, int stopSec,
				   int baseYYMMDD, int baseSec,
				   int currYYMMDD, int currSec);

extern MF_TimeMgr MF_TimeMgrNewRestartReadIS(MF_CalendarType type,
					     int nstep,
					     int stepDays, int stepSec,
					     int startYYMMDD, int startSec,
					     int stopYYMMDD, int stopSec,
					     int baseYYMMDD, int baseSec,
					     int currYYMMDD, int currSec);

extern void MF_TimeMgrDelete(MF_TimeMgr this);

#endif










