!===============================================================================
! MFM_Alarm Class
!===============================================================================
!
! Alarms are part of the Modeling Framework Time Management module 
! (MFM_TimeMgmt).  An alarm object can be activated at periodic, monthly, or 
! yearly intervals. 
!
!     Public Member Functions:
!     ------------------------
!     MFM_AlarmInitPeriodic     
!     MFM_AlarmInitMonthly
!     MFM_AlarmInitYearly
!     MFM_AlarmGetType
!     MFM_AlarmSet
!     MFM_AlarmIsOn
!
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmInitPeriodic
!
! !INTERFACE:
      function MFM_AlarmInitPeriodic(period, offset, rc)
    
! !RETURN VALUE:
      type(MFM_Alarm) :: MFM_AlarmInitPeriodic    ! returned alarm object
    
! !PARAMETERS:
      type(MFM_Time), intent(in) :: period        ! time period
      type(MFM_Time), intent(in) :: offset        ! time offset
      integer, intent(out), optional :: rc        ! return code 

! !DESCRIPTION:
!     Returns a new alarm.  
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmInitPeriodic(MFM_AlarmInitPeriodic, period, 
     &  offset, stub)
      if (present(rc)) rc = stub

      end function MFM_AlarmInitPeriodic

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmInitMonthly
!
! !INTERFACE:
      function MFM_AlarmInitMonthly(rc)
    
! !RETURN VALUE:
      type(MFM_Alarm) :: MFM_AlarmInitMonthly       ! returned alarm object
    
! !PARAMETERS:
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns a new alarm object.  
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmInitMonthly(MFM_AlarmInitMonthly, stub)
      if (present(rc)) rc = stub

      end function MFM_AlarmInitMonthly

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmInitYearly
!
! !INTERFACE:
      function MFM_AlarmInitYearly(rc)

! !RETURN VALUE:    
      type(MFM_Alarm) :: MFM_AlarmInitYearly        ! returned alarm object
    
! !PARAMETERS:
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns a new alarm object.  
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmInitYearly(MFM_AlarmInitYearly, stub)
      if (present(rc)) rc = stub

      end function MFM_AlarmInitYearly

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmGetType
!
! !INTERFACE:
      function MFM_AlarmGetType(this, rc)
    
! !RETURN VALUE:
      integer :: MFM_AlarmGetType                   ! returned type

! !PARAMETERS:
      type(MFM_Alarm), intent(in) :: this           ! alarm object    
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns true if the alarm is on, false if not.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmGetType(this, MFM_AlarmGetType, stub)
      if (present(rc)) rc = stub

      end function MFM_AlarmGetType

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmIsOn
!
! !INTERFACE:
      function MFM_AlarmIsOn(this, timeMgr, rc)
    
! !RETURN VALUE:
      logical :: MFM_AlarmIsOn                      ! true if alarm is on

! !PARAMETERS:
      type(MFM_Alarm), intent(in) :: this           ! alarm object    
      type(MFM_TimeMgr), intent(in) :: timeMgr      ! time manager
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns true if the alarm is on, false if not.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmIsOn(this, timeMgr, MFM_AlarmIsOn, stub)
      if (present(rc)) rc = stub

      end function MFM_AlarmIsOn

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_AlarmSet
!
! !INTERFACE:
      subroutine MFM_AlarmSet(this, alarmOn, rc)
    
! !PARAMETERS:
      type(MFM_Alarm), intent(inout) :: this        ! alarm object    
      logical, intent(in) :: alarmOn                ! true if alarm on
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Sets whether the alarm is on.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_AlarmSet(this, alarmOn, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_AlarmSet

!===============================================================================














