! $Id: MFM_Date.F,v 1.3.2.1 2001/11/09 18:44:32 erik Exp $
!===============================================================================
! MFM_Date Class
!===============================================================================
!
! Dates are part of the Modeling Framework Time Managment module (MFM_TimeMgmt).  
! A date object stores the attributes of a calendar date, including year, 
! month, day, and time of day.  Gregorian (MFM_GREGORIAN) and no leap year 
! (MFM_NO_LEAP) calendars are supported.
!
!     Public Member Functions:
!     ------------------------
!     MFM_DateInit (Interface only)
!     MFM_DateSet (Interface only)
!     MFM_DateGet (Interface only)
!     MFM_DateGetDayOfYear
!     MFM_DateGetFltDayOfYear
!     MFM_DateGetCalendarType
!     MFM_DateCopy 
!     MFM_DateDiff
!     MFM_DateIsLater
!     MFM_DateIncrement
!     MFM_DateIncrementSec
!     MFM_DateIncrementDay
!     MFM_DateIncrementMonth
!     MFM_DateIncrementYear
!     MFM_DateDecrement
!     MFM_DatePrint
!
!     Private Member Functions:
!     -------------------------
!     MFM_DateInitIS (overloads MFM_DateInit)
!     MFM_DateInitUndefined (overloads MFM_DateInit)
!     MFM_DateCopyInit (overloads MFM_DateInit)
!     MFM_DateSetIS (overloads MFM_DateSet)
!     MFM_DateGetIS (overloads MFM_DateGet)
!
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateInitIS
!
! !INTERFACE:
      function MFM_DateInitIS(type, yearmmdd, tod, rc)
    
! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateInitIS            ! returned date object
    
! !PARAMETERS:
      integer, intent(in) :: type                 ! MFM_GREGORIAN or MFM_NO_LEAP
      integer, intent(in) :: yearmmdd             ! calendar date
      integer, intent(in) :: tod                  ! time of day in seconds
      integer, intent(out), optional :: rc        ! return code 

! !DESCRIPTION:
!     Initializes a {\tt MFM\_Date} object.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitIS(MFM_DateInitIS, type, yearmmdd, tod, stub)
      if (present(rc)) rc = stub
    
      end function MFM_DateInitIS

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateInitUndefined
!
! !INTERFACE:
      function MFM_DateInitUndefined(rc)
    
! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateInitUndefined       ! returned date object
    
! !PARAMETERS:
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Initializes a {\tt MFM\_Date} object wih undefined contents.  The
!     values of internal attributes are set to {\tt MFM\_TIME\_UNDEFINED}.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateInitUndefined, stub)
      if (present(rc)) rc = stub
 
      end function MFM_DateInitUndefined

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateCopyInit
!
! !INTERFACE:
      function MFM_DateCopyInit(orig, rc)
    
! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateCopyInit            ! returned date object
    
! !PARAMETERS:
      type(MFM_Date), intent(in) :: orig            ! original date
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Initializes a new date object to the contents of another date.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateCopyInit(MFM_DateCopyInit, orig, stub) 
      if (present(rc)) rc = stub

      end function MFM_DateCopyInit

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateSetIS
!
! !INTERFACE:
      subroutine MFM_DateSetIS(date, type, yearmmdd, tod, rc)

! !PARAMETERS:
      type(MFM_Date), intent(out) :: date           ! date object
      integer, intent(in) :: type                   ! MFM_GREGORIAN or MFM_NO_LEAP
      integer, intent(in) :: yearmmdd               ! calendar date
      integer, intent(in) :: tod                    ! time of day
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Sets the attributes of a date object.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateSetIS(date, type, yearmmdd, tod, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_DateSetIS

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateGetIS
!
! !INTERFACE:
      subroutine MFM_DateGetIS(date, yearmmdd, tod, rc)

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! date object
      integer, intent(out) :: yearmmdd              ! calendar date
      integer, intent(out) :: tod                   ! time of day
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns the attributes of a {\tt date}.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateGetIS(date, yearmmdd, tod, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_DateGetIS

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateGetDayOfYear
!
! !INTERFACE:
      function MFM_DateGetDayOfYear(date, rc)

! !RETURN VALUE:
      integer :: MFM_DateGetDayOfYear               ! returned day of year

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! date object      
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns the day of the year corresponding to {\tt date}.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateGetDayOfYear(date, MFM_DateGetDayOfYear, stub)
      if (present(rc)) rc = stub

      end function MFM_DateGetDayOfYear

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DateGetFltDayOfYear
!
! !INTERFACE:
      function MFM_DateGetFltDayOfYear(date, rc)
! !RETURN VALUE:
      real(8) :: MFM_DateGetFltDayOfYear ! Returns calendar day as intdays.seconds
! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! date
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns the calendar day as a float.  The day of year is returned as
!     the integer part and the seconds/milliseconds are returned in the 
!     fractional part.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateGetFltDayOfYear(date, MFM_DateGetFltDayOfYear, stub)
      if (present(rc)) rc = stub

      end function MFM_DateGetFltDayOfYear

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateGetCalendarType
!
! !INTERFACE:
      function MFM_DateGetCalendarType(date, rc)

! !RETURN VALUE:
      integer :: MFM_DateGetCalendarType            ! returned calendar type

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! date object
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Returns the calendar type of {\tt date}.  Valid values are:
!     MFM\_CALENDAR\_TYPE\_UNDEFINED = 0, MFM\_NO\_LEAP = 1, 
!     MFM\_GREGORIAN = 2, MFM\_360\_DAY = 3
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateGetCalendarType(date, MFM_DateGetCalendarType, stub)
      if (present(rc)) rc = stub

      end function MFM_DateGetCalendarType

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateCopy
!
! !INTERFACE:
      subroutine MFM_DateCopy(date, orig, rc)

! !PARAMETERS:
      type(MFM_Date), intent(out) :: date           ! copy of original date
      type(MFM_Date), intent(in) :: orig            ! original date
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Copies the date {\tt orig} to {\tt date}.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateCopy(date, orig, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_DateCopy

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateDiff
!
! !INTERFACE:
      subroutine MFM_DateDiff(earlyDate, lateDate, diff, isLater, rc)

! !PARAMETERS:
      type(MFM_Date), intent(in) :: earlyDate      ! earlier date
      type(MFM_Date), intent(in) :: lateDate       ! later date
      type(MFM_Time), intent(out) :: diff          ! time difference between dates
      logical, intent(out) :: isLater              ! true if late date is 
                                                   !   actually later
      integer, intent(out), optional :: rc         ! return code

! !DESCRIPTION:
!     Determines the time difference between two dates and returns a 
!     logical indicator as to which date is later.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub;

      call MF_DateDiff(earlyDate, lateDate, diff, isLater, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_DateDiff

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateIsLater
!
! !INTERFACE:
       subroutine MFM_DateIsLater(earlyDate, lateDate, isLater, rc)
    
! !PARAMETERS:                              
       type(MFM_Date), intent(in) :: earlyDate      ! earlier date
       type(MFM_Date), intent(in) :: lateDate       ! later date
       logical, intent(out) :: isLater              ! true if late date is
                                                    !   actually later
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!      Returns true if {\tt lateDate} is later than {\tt earlyDate}.
!
!EOP
!-------------------------------------------------------------------------------

       integer stub

       call MF_DateIsLater(earlyDate, lateDate, isLater, stub)
       if (present(rc)) rc = stub

       end subroutine MFM_DateIsLater

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateIncrement
!
! !INTERFACE:  
      function MFM_DateIncrement(date, time, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateIncrement           ! returns incremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! original date     
      type(MFM_Time), intent(in) :: time            ! time increment
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Increments {\tt date} by {\tt time} and returns the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateIncrement, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateIncrement(date, MFM_DateIncrement, time, stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateIncrement

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DateIncrementSec
!
! !INTERFACE:
      function MFM_DateIncrementSec(date, nsecs, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateIncrementSec        ! returns incremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! original date
      integer, intent(in) :: nsecs                  ! time increment in seconds
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Increments {\tt date} by {\tt nsecs} seconds and returns
!     the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateIncrementSec, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateIncrementSec(date, MFM_DateIncrementSec, nsecs, 
     &    stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateIncrementSec

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DateIncrementDay
!
! !INTERFACE:
      function MFM_DateIncrementDay(date, ndays, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateIncrementDay     ! returns incremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date         ! original date
      integer, intent(in) :: ndays               ! time increment in days
      integer, intent(out), optional :: rc       ! return code

! !DESCRIPTION:
!     Increments {\tt date} by {\tt ndays} days and returns
!     the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateIncrementDay, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateIncrementDay(date, MFM_DateIncrementDay, ndays, 
     &    stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateIncrementDay

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DateIncrementMonth
!
! !INTERFACE:
      function MFM_DateIncrementMonth(date, nmonths, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateIncrementMonth   ! returns incremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date         ! original date
      integer, intent(in) :: nmonths             ! time increment in months
      integer, intent(out), optional :: rc       ! return code

! !DESCRIPTION:
!     Increments {\tt date} by {\tt nmonths} months and returns
!     the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateIncrementMonth, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateIncrementMonth(date, MFM_DateIncrementMonth, 
     &    nmonths, stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateIncrementMonth

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DateIncrementYear
!
! !INTERFACE:
      function MFM_DateIncrementYear(date, nyears, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateIncrementYear    ! returns incremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date         ! original date
      integer, intent(in) :: nyears              ! time increment in years
      integer, intent(out), optional :: rc       ! return code

! !DESCRIPTION:
!     Increments {\tt date} by {\tt nyears} years and returns
!     the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateIncrementYear, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateIncrementYear(date, MFM_DateIncrementYear, nyears, 
     &    stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateIncrementYear

!===============================================================================
!BOP
!
! !IROUTINE:  MFM_DateDecrement
!
! !INTERFACE:  
      function MFM_DateDecrement(date, time, rc)

! !RETURN VALUE:
      type(MFM_Date) :: MFM_DateDecrement         ! returns decremented date

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date          ! original date     
      type(MFM_Time), intent(in) :: time          ! time increment
      integer, intent(out), optional :: rc        ! return code

! !DESCRIPTION:
!     Decrements {\tt date} by {\tt time} and returns the result.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DateInitUndefined(MFM_DateDecrement, stub)
      if (stub == MFM_SUCCESS) then
        call MF_DateDecrement(date, MFM_DateDecrement, time, stub)
      end if
      if (present(rc)) rc = stub

      end function MFM_DateDecrement

!===============================================================================
!BOP
! 
! !IROUTINE:  MFM_DatePrint
!
! !INTERFACE:
      subroutine MFM_DatePrint(date, rc)

! !PARAMETERS:
      type(MFM_Date), intent(in) :: date            ! date
      integer, intent(out), optional :: rc          ! return code

! !DESCRIPTION:
!     Prints the contents of {\tt date}.
!
!EOP
!-------------------------------------------------------------------------------

      integer stub

      call MF_DatePrint(date, stub)
      if (present(rc)) rc = stub

      end subroutine MFM_DatePrint


!===============================================================================
