!===============================================================================
      module MFM_TimeMgmtTypes

! This file is a general header for the Fortran interface to the
! Modeling Framework Modules.  It contains definitions of derived types
! and common parameters.  
!
! IMPORTANT NOTE:  These types are defined in detail ONLY in order to correctly 
!                  allocate enough memory for the C MF library to build data
!                  structures within them.  They CANNOT be accessed directly 
!                  via the FORTRAN % dereference operator. 
!
!===============================================================================

      type MFM_TOD
        private
        sequence
        integer(8) type
        integer(8) sec
        integer(8) msec
      end type MFM_TOD

      type MFM_Calendar
        private
        sequence
        integer(8) type
        integer(8) dim(13)
        integer(8) dimRunningSum(13)
        integer(8) diy
      end type MFM_Calendar

      type MFM_Time
        private
        sequence
        integer(8) day
        type(MFM_TOD) tod
      end type MFM_Time

      type MFM_Date
        private
        sequence
        type(MFM_Calendar) calendar
        integer(8) year
        integer(8) month
        integer(8) day
        type(MFM_TOD) tod
        integer(8) julianDay
        integer(8) dayOfYear
      end type MFM_Date

      type MFM_TimeMgr
        private
        sequence
        integer(8) nstep
        type(MFM_Time) stepSize
        type(MFM_Date) startDate
        type(MFM_Date) stopDate
        type(MFM_Date) baseDate
        type(MFM_Date) currDate
        type(MFM_Date) prevDate
      end type MFM_TimeMgr

      type MFM_Alarm
        private
        sequence
        integer(8) type
        integer(8) offset
        integer(8) period
        integer(8) alarmOn
      end type MFM_Alarm

      integer, parameter :: MFM_SUCCESS=0
        
      integer, parameter :: MFM_TIME_UNDEFINED=-1

      integer, parameter :: MFM_CALENDAR_TYPE_UNDEFINED=0,
     &                      MFM_NO_LEAP=1,
     &                      MFM_GREGORIAN=2,                    
     &                      MFM_360_DAY=3

      integer, parameter :: MFM_ALARM_PERIODIC=0,
     &                      MFM_ALARM_MONTHLY=1,
     &                      MFM_ALARM_YEARLY=2

      integer, parameter :: MFM_ERR_RETURN=0,
     &                      MFM_ERR_EXIT=1,
     &                      MFM_ERR_USER_DEFINED=2

      end module MFM_TimeMgmtTypes










