!===============================================================================
! MFM_TimeMgrOverload.F
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrInit
!
! !INTERFACE:
      interface MFM_TimeMgrInit
        module procedure MFM_TimeMgrInitStd,
     &                   MFM_TimeMgrInitNoBaseStd,
     &                   MFM_TimeMgrInitIS,
     &                   MFM_TimeMgrInitNoBaseIS
      end interface

! !DESCRIPTION:
!     Initializes a new {\tt MFM\_TimeMgr} object. \\
!     See the explicit interfaces: 
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrInitStd}       & general interface to init a time manager \\
!     {\tt MFM\_TimeMgrInitNoBaseStd} & general interface to init a time manager,
!                                       with base date == start date \\
!     {\tt MFM\_TimeMgrInitIS}        & init a time manager with integer second 
!                                       discretization \\
!     {\tt MFM\_TimeMgrInitNoBaseIS}  & init a time manager with integer second
!                                       discretization, base date == start date \\
!     \end{tabular}  
!
! SEE ALSO:
!   {\tt MFM\_TimeMgrInitStd, MFM\_TimeMgrInitNoBaseStd, MFM\_TimeMgrInitIS, 
!   MFM\_TimeMgrInitNoBaseIS}
!
!EOP
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrGetStepSize
!
! !INTERFACE:
      interface MFM_TimeMgrGetStepSize
        module procedure MFM_TimeMgrGetStepSizeStd,
     &                   MFM_TimeMgrGetStepSizeIS
      end interface

! !DESCRIPTION:
!     Gets a step size. \\
!     See the explicit interfaces:
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrGetStepSizeStd} & get step size as time type \\
!     {\tt MFM\_TimeMgrGetStepSizeIS}  & get step size as days and seconds \\
!     \end{tabular}  
!
! SEE ALSO:
!   MFM\_TimeMgrGetStepSizeStd, MFM\_TimeMgrGetStepSizeIS
!
!EOP
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrSetStepSize
!
! !INTERFACE:
      interface MFM_TimeMgrSetStepSize
        module procedure MFM_TimeMgrSetStepSizeStd,
     &                   MFM_TimeMgrSetStepSizeIS
      end interface

! !DESCRIPTION:
!     Sets a step size. \\
!     See the explicit interfaces:
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrSetStepSizeStd}  & set step size with time type \\
!     {\tt MFM\_TimeMgrSetStepSizeIS}   & set step size with days and seconds \\
!     \end{tabular}  
!
! SEE ALSO:
!   {\tt MFM\_TimeMgrSetStepSizeStd, MFM\_TimeMgrSetStepSizeIS}
!
!EOP
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrRestartWrite
!
! !INTERFACE:
      interface MFM_TimeMgrRestartWrite
        module procedure MFM_TimeMgrRestartWriteIS
      end interface

! !DESCRIPTION:
!     Returns the data needed for a restart. \\
!     See the explicit interfaces:
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrRestartWriteIS}   & return restart data \\
!     \end{tabular}  
!
! SEE ALSO:
!   {\tt MFM\_TimeMgrRestartWrite}
!
!EOP
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrRestartRead
!
! !INTERFACE:
      interface MFM_TimeMgrRestartRead
        module procedure MFM_TimeMgrRestartReadIS
      end interface

! !DESCRIPTION:
!     Retrieves the restart data and returns a TimeMgr. \\
!     See the explicit interfaces:
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrRestartReadIS}   & constructs restart TimeMgr (int) seconds  \\
!     \end{tabular}  
!
! SEE ALSO:
!   {\tt MFM\_TimeMgrRestartRead}
!
!EOP
!===============================================================================
!BOP
!
! !IROUTINE:  MFM_TimeMgrSetCurrDate
!
! !INTERFACE:
      interface MFM_TimeMgrSetCurrDate
        module procedure MFM_TimeMgrSetCurrDateIS
      end interface
! !DESCRIPTION:
!     Sets the current date (updates prev date).
!     See the explicit interfaces:
!
!     \begin{tabular}{p{2in}p{3.5in}}
!     {\tt MFM\_TimeMgrSetCurrDateIS}   & set curr date \\
!     \end{tabular}  
!
! SEE ALSO:
!   {\tt MFM\_TimeMgrGetCurrDate}
!
!EOP
!===============================================================================
