/* MF_Date.c */

#include "MF_Date.h"

/*--------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateNewIS"

MF_Date MF_DateNewIS(MF_CalendarType type, int yearmmdd, int seconds) 
{
  MF_Date date = MF_NULL;

  date = (MF_Date) malloc (sizeof(struct DateClass));
  if(!date) return(MF_NULL);

  MF_DateConstructIS(date, type, yearmmdd, seconds);

  return(date);
}



/*--------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateNewUndefined"

MF_Date MF_DateNewUndefined() 
{
  MF_Date date = MF_NULL;

  date = (MF_Date) malloc (sizeof(struct DateClass));
  if(!date) return(MF_NULL);

  MF_DateConstructUndefined(date);

  return(date);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateConstructIS"

int MF_DateConstructIS(MF_Date this, MF_CalendarType type, int yearmmdd, 
  int seconds)
{
  int year, month, day;

  if((type == MF_CALENDAR_TYPE_UNDEFINED) || (yearmmdd == MF_TIME_UNDEFINED) || 
     (seconds == MF_TIME_UNDEFINED)){
    MF_DateConstructUndefined(this);
    /* For cases in which the user wants to set the date's calendar type only. */
    this->calendar.type = type;
    return(MF_SUCCESS);
  }

  year = yearmmdd/10000;
  month = (yearmmdd%10000)/100;
  day = yearmmdd%100;

  MF_CalendarConstruct(&this->calendar, type, year);
  MF_TODConstructIS(&this->tod, seconds);

#ifdef MF_DEBUG
  if((day < 1) || (day > this->calendar.dim[month])){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE, 0, "day = %d", day);
  }

  if((month < 1) || (month > 12)){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE,0, "month = %d", month);
  }
#endif

  /* Compute the Julian day (absolute reference day). */

  MF_DATE_2_JULIAN_DAY(this->julianDay, year, month, day);

  MF_CalendarBuildDayOfYear(&this->calendar, month, day, &this->dayOfYear);

  this->year = year;
  this->month = month;
  this->day = day;

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateConstructUndefined"

int MF_DateConstructUndefined(MF_Date this)
{
  this->julianDay = MF_TIME_UNDEFINED;
  this->dayOfYear = MF_TIME_UNDEFINED;

  this->year = MF_TIME_UNDEFINED;
  this->month = MF_TIME_UNDEFINED;
  this->day = MF_TIME_UNDEFINED;
  MF_CalendarConstructUndefined(&this->calendar);
  MF_TODConstructUndefined(&this->tod);
    
  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateCopyConstruct"

int MF_DateCopyConstruct(MF_Date this, MF_Date orig)
{
  this->calendar = orig->calendar;
  this->tod = orig->tod;
  this->year = orig->year;
  this->month = orig->month;
  this->day = orig->day;
  this->julianDay = orig->julianDay;
  this->dayOfYear = orig->dayOfYear;

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateCopy"

int MF_DateCopy(MF_Date this, MF_Date orig)
{
#ifdef MF_DEBUG

  if((this->calendar.type != orig->calendar.type) &&
     (this->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }

#endif

  MF_DateCopyConstruct(this, orig);

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateSetIS"

int MF_DateSetIS (MF_Date this, MF_CalendarType type, int yearmmdd, int seconds)
{
#ifdef MF_DEBUG
  if((MF_TIME_NEGATIVE_ATTR_IS(yearmmdd, seconds)) ||
     (MF_TIME_UNDEFINED_ATTR_IS(yearmmdd, seconds))){
    MF_ERRA2(MF_ERR_ARG_OUTOFRANGE, 0, 
    "yearmmdd or seconds are invalid or undefined: yearmmdd = %d, seconds = %d",
    yearmmdd, seconds);
  }
#endif

  MF_DateConstructIS (this, type, yearmmdd, seconds);

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateGetIS"

int MF_DateGetIS(MF_Date this, int *yearmmdd, int *seconds)
{

  *yearmmdd = this->year*10000 + this->month*100 + this->day;
  *seconds = this->tod.sec;

  return(MF_SUCCESS);		 
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateGetDayOfYear"

int MF_DateGetDayOfYear(MF_Date this, int *dayOfYear)
{
  *dayOfYear = this->dayOfYear;

  return(MF_SUCCESS);		 
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateGetCalendarType"

int MF_DateGetCalendarType(MF_Date this, MF_CalendarType *type)
{
  *type = this->calendar.type;
  
  return (MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_Date2Str"

int MF_Date2Str(MF_Date this, char *str)
{
  char strLocal[32];

  /* Implementation */

  return(MF_SUCCESS);		 
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIncrement"

int MF_DateIncrement(MF_Date this, MF_Date incDate, MF_Time time)
{
  int year, month, day;
  int ndays;

#ifdef MF_DEBUG
  if((this->calendar.type != incDate->calendar.type) &&
     (incDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }
#endif

  /* Add the times of day for the original date and time increment 
     and return the result as a new time of day and a number of days. */

  MF_TODIncrement(&this->tod, &time->tod, &incDate->tod, &ndays);
  ndays += time->day;
  incDate->julianDay = this->julianDay + ndays;

  switch(this->calendar.type){
    case MF_GREGORIAN:
      MF_JULIAN_DAY_2_DATE(incDate->julianDay, year, month, day);
      break;
    case MF_NO_LEAP:
      month = this->month;  
      year = this->year+(ndays/this->calendar.diy);
      day = this->day+(ndays%this->calendar.diy);

      while(day>this->calendar.dim[month]){
        day-=this->calendar.dim[month];
        month+=1;
        if(month==13){
          ++year;
          month=1;
        }
      }
      break;
    default:
      MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	      "calendar type is not supported by this method");
  }
        
  incDate->year = year;
  incDate->month = month;
  incDate->day = day;

  MF_CalendarConstruct(&incDate->calendar, this->calendar.type, year);
  MF_CalendarBuildDayOfYear(&incDate->calendar, month, day, &incDate->dayOfYear);

  return(MF_SUCCESS);		 
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateDecrement"

int MF_DateDecrement(MF_Date this, MF_Date decDate, MF_Time time)
{
  int year, month, day;
  int ndays;

#ifdef MF_DEBUG
  if((this->calendar.type != decDate->calendar.type) &&
     (decDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }
#endif

  /* Add the times of day for the original date and time increment 
     and return the result as a new time of day and a number of days. */

  MF_TODDecrement(&this->tod, &time->tod, &decDate->tod, &ndays);
  ndays += time->day;
  decDate->julianDay = this->julianDay - ndays;

  switch(this->calendar.type){
    case MF_GREGORIAN:
      MF_JULIAN_DAY_2_DATE(decDate->julianDay, year, month, day);
      break;
    case MF_NO_LEAP:
      month = this->month;  
      year = this->year-(ndays/this->calendar.diy);
      day = this->day-(ndays%this->calendar.diy); /* days may be negative at this point */

      while(day<=0){
        month-=1;
        if(month==0){
          --year;
          month=12;
        }
        day+=this->calendar.dim[month];
      }
      break;
    default:
      MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	      "calendar type is not supported by this method");
  }
        
  decDate->year = year;
  decDate->month = month;
  decDate->day = day;

  MF_CalendarConstruct(&decDate->calendar, this->calendar.type, year);
  MF_CalendarBuildDayOfYear(&decDate->calendar, month, day, &decDate->dayOfYear);

  return(MF_SUCCESS);		 
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIncrementSec"

int MF_DateIncrementSec(MF_Date this, MF_Date incDate, int nseconds)
{
  MF_TimeClass time;

#ifdef MF_DEBUG

  if((this->calendar.type != incDate->calendar.type) &&
     (incDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }

  if((nseconds < 0) || (nseconds == MF_TIME_UNDEFINED)){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE, 0, 
    "seconds are invalid or undefined: seconds = %d", nseconds);
  }
#endif
  
  switch(this->tod.type){
    case MF_TOD_INT_SEC:
      MF_TimeConstructIS(&time, 0, nseconds);
      break;
    default:
      MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	      "time of day type is not supported by this method");
  }

  MF_DateIncrement(this, incDate, &time);

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIncrementDay"

int MF_DateIncrementDay (MF_Date this, MF_Date incDate, int ndays)
{
  MF_TimeClass time;

#ifdef MF_DEBUG

  if((this->calendar.type != incDate->calendar.type) &&
     (incDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }

  if((ndays < 0) || (ndays == MF_TIME_UNDEFINED)){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE, 0, 
    "days are invalid or undefined: days = %d", ndays);
  }
#endif
  
  switch(this->tod.type){
    case MF_TOD_INT_SEC:
      MF_TimeConstructIS(&time, ndays, 0);
      break;
    default:
      MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	      "time of day type is not supported by this method");
  }

  MF_DateIncrement(this, incDate, &time);

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIncrementMonth"

int MF_DateIncrementMonth (MF_Date this, MF_Date incDate, int nmonths)
{

#ifdef MF_DEBUG

  if((this->calendar.type != incDate->calendar.type) &&
     (incDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }

  if((nmonths < 0) || (nmonths == MF_TIME_UNDEFINED)){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE, 0, 
    "months are invalid or undefined: months = %d", nmonths);
  }
#endif

  MF_DateCopy(incDate, this);

  incDate->month += nmonths;
  
  /* Cycle to a new year if needed. */

  if(incDate->month > 12){
    incDate->year += (incDate->month)/12; 
    incDate->month = (incDate->month)%12;

    /* If in a new year, rebuild the calendar. */

    MF_CalendarConstruct(&incDate->calendar, incDate->calendar.type, incDate->year);
  }

  /* If the current day exceeds the number of days in the month, set the
     day of the month to the last day of the month. Seconds will remain the
     same.  Flag if this happens when in DEBUG mode. */

  if(incDate->day > incDate->calendar.dim[incDate->month]){
#ifdef DEBUG
    MF_ERRA(MF_ERR_DATE, 0,"incrementing by a month resulted in an invalid day");    
#endif
    incDate->day = incDate->calendar.dim[incDate->month];
  }

  /* Compute the new Julian day. */

  MF_DATE_2_JULIAN_DAY(incDate->julianDay, incDate->year, incDate->month, incDate->day);
  
  /* Compute the new day of the year */  

  MF_CalendarBuildDayOfYear(&incDate->calendar, incDate->month, incDate->day, 
    &incDate->dayOfYear);

  return(MF_SUCCESS);
}  



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIncrementYear"

int MF_DateIncrementYear (MF_Date this, MF_Date incDate, int nyears)
{
#ifdef MF_DEBUG

  if((this->calendar.type != incDate->calendar.type) &&
     (incDate->calendar.type != MF_CALENDAR_TYPE_UNDEFINED)){
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	    "argument calendar types are different or uninitialized");
  }

  if((nyears < 0) || (nyears == MF_TIME_UNDEFINED)){
    MF_ERRA1(MF_ERR_ARG_OUTOFRANGE, 0, 
    "years are invalid or undefined: years = %d", nyears);
  }
#endif

  MF_DateCopy(incDate, this);

  incDate->year+=nyears;
  
  MF_CalendarConstruct(&incDate->calendar, incDate->calendar.type, incDate->year);

  /* If the current day exceeds the number of days in the month, set the
     day of the month to the last day of the month. Seconds will remain the
     same.  Flag if this happens when in DEBUG mode. */

  if(incDate->day > incDate->calendar.dim[incDate->month]){
#ifdef DEBUG
    MF_ERRA(MF_ERR_DATE, 0,"incrementing by a year resulted in an invalid day");    
#endif
    incDate->day = incDate->calendar.dim[incDate->month];
  }

  /* Compute the new Julian day. */

  MF_DATE_2_JULIAN_DAY(incDate->julianDay, incDate->year, incDate->month, incDate->day);
  
  /* Compute the new day of the year */  

  MF_CalendarBuildDayOfYear(&incDate->calendar, incDate->month, incDate->day, 
    &incDate->dayOfYear);

  return(MF_SUCCESS);
}  



/*----------------------------------------------------------------------------*/

#undef __FUNC__
#define __FUNC__ "MF_DateDiff"

int MF_DateDiff (MF_Date earlyDate, MF_Date lateDate, MF_Time diff, 
		  MF_Bool *isLater)
{
  int extraday, earlyJulDay, lateJulDay;
  MF_DateClass tmpEarly, tmpLate;
  MF_TimeClass earlyTime, lateTime;

#ifdef MF_DEBUG

  /* Check that date arguments are using the same calendar. */

  if(earlyDate->calendar.type != lateDate->calendar.type)
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, "dates must use the same calendar");

#endif

  switch(earlyDate->calendar.type){
    case MF_GREGORIAN:

      MF_DATE_2_JULIAN_DAY(earlyJulDay, earlyDate->year, earlyDate->month,
        earlyDate->day);    
  
      MF_DATE_2_JULIAN_DAY(lateJulDay, lateDate->year, lateDate->month,
        lateDate->day);

      MF_TimeConstruct(&earlyTime, earlyJulDay, &earlyDate->tod);
      MF_TimeConstruct(&lateTime, lateJulDay, &lateDate->tod);
      MF_TimeDiff(&earlyTime, &lateTime, diff, isLater);
      break;
    case MF_NO_LEAP:
      MF_DateIsLater(earlyDate, lateDate, isLater); 
      if(*isLater){
        MF_DateCopyConstruct(&tmpEarly, earlyDate);
	MF_DateCopyConstruct(&tmpLate, lateDate);
      }
      else{
        MF_DateCopyConstruct(&tmpEarly, lateDate);
	MF_DateCopyConstruct(&tmpLate, earlyDate);
      }
      MF_TODDecrement(&tmpLate.tod, &tmpEarly.tod, &diff->tod, &extraday);
      if(extraday == 1) MF_DateIncrementDay(&tmpEarly, &tmpEarly, 1); 
      diff->day = tmpLate.dayOfYear - tmpEarly.dayOfYear;
      if(diff->day < 0){
        diff->day += tmpLate.calendar.diy;
        ++(tmpEarly.year);
      }
      diff->day += (tmpLate.year - tmpEarly.year)*365;
      break;  
    default:
      MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, 
	      "calendar type is not supported by this method");
  }
    
  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateIsLater"

int MF_DateIsLater (MF_Date earlyDate, MF_Date lateDate, MF_Bool *isLater)
{
  MF_Bool todIsLater;
#ifdef MF_DEBUG


  /* Check that date arguments are using the same calendar. */

  if(earlyDate->calendar.type != lateDate->calendar.type)
    MF_ERRA(MF_ERR_ARG_OUTOFRANGE, 0, "dates must use the same calendar");

#endif

  /* If none of these complete conditions was satisfied, the result is false. */
  *isLater= MF_FALSE;
  
  if(lateDate->year > earlyDate->year)
    {
      /* Year is larger, so done */
      *isLater = MF_TRUE;
    }
  else if(lateDate->year == earlyDate->year)
    {
      /* Years are equal, so further testing required */
      if (lateDate->dayOfYear > earlyDate->dayOfYear)
	{
	  /* Year ==, Day is larger, so done. */
	  *isLater = MF_TRUE;
	}
      else if(lateDate->dayOfYear == earlyDate->dayOfYear)      
	{
	  /* Year==, Day ==, so test time of day */
	  MF_TODIsLater(&earlyDate->tod, &lateDate->tod, &todIsLater);
	  if(todIsLater)
	    {
	      /* Time of day larger, done */
	      *isLater=MF_TRUE;
	    }
	} /* Days == */
    } /* Years == */
  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DatePrint"

int MF_DatePrint(MF_Date this)
{
  printf("Printing Date:\n");
  printf("year      = %d\n",  this->year);
  printf("month     = %d\n",  this->month);
  printf("day       = %d\n",  this->day);
  printf("julianDay = %d\n", this->julianDay);
  printf("dayOfYear = %d\n", this->dayOfYear); 

  printf("Printing Date internal Calendar:\n");
  MF_CalendarPrint(&this->calendar);

  printf("Printing Date internal TOD:\n");
  MF_TODPrint(&this->tod);

  return(MF_SUCCESS);
}



/*----------------------------------------------------------------------------*/
int MF_DateGetFltDayOfYear(MF_Date this, double *day)
{
  int iday;
  int ret = MF_SUCCESS;
  
  ret = MF_DateGetDayOfYear(this, &iday);

  *day = (double) iday;

  /* e.g. 0.1 days = 2.4 hours = 8640 seconds */
  /* Handle the integer seconds and the normal case here.  this->tod.msec
     turns out to be -1, not zero == MF_TIME_UNDEFINED */
  if (this->tod.msec == MF_TIME_UNDEFINED)
  {
    *day += (this->tod.sec)
      / (double) (MF_SID) ; 
  }
  else
  {
    *day += (this->tod.sec + (this->tod.msec / 1000.0) )
      / (double) (MF_SID) ; 
  }

  return ret;
}


/*----------------------------------------------------------------------------*/
#undef __FUNC__
#define __FUNC__ "MF_DateDelete"

void MF_DateDelete (MF_Date this)
{
  free(this);
  
  return;
}
